package simulation.log;
import java.util.ArrayList;

import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;




import desmoj.core.simulator.*;


public class LogProcess extends SimProcess {
	
	public LogProcess(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
	}

	private double fixRange(double i){
		if (i<0) return 0;
		else if (i>1) return 1;
		else return i;
	}
	
	@Override
	public void lifeCycle() {
		ModelS model = (ModelS) getModel();
		ArrayList<Node> nodes = model.getNodes();
		
		double lastTime = currentTime().getTimeValue();
		double currTime;
		double lastCPU[] = new double[nodes.size()];
		double lastMem[] = new double[nodes.size()];
		double lastDisk[] = new double[nodes.size()];
		double lastEth[] = new double[nodes.size()];
		double lastread[] =  new double[nodes.size()];
		double lastrecv[] = new double[nodes.size()];
		double lastsend[] = new double[nodes.size()];
		
		double cpu, mem, disk, eth, dtime, dcpu, dmem, ddisk, deth, rdbw, sdbw, rvbw, send, recv, read;
		while(true){
			hold(new SimTime(Config.LOG_SAMPLE_TIME));
			currTime=currentTime().getTimeValue();
			dtime=currTime-lastTime;
			for (int i=0; i< nodes.size(); i++){
				Node node=nodes.get(i);
				
				cpu = node.getCPU().avgUsage();
				mem = node.getMemory().avgUsage();
				disk = node.getDisk().avgUsage();
				eth = node.getEth().avgUsage();
				read = node.getDiskread();
				recv = node.getEthrecv();
				send = node.getEthsend();
				
				dcpu = fixRange((cpu*currTime-lastCPU[i]*lastTime)/dtime);
				dmem = fixRange((mem*currTime-lastMem[i]*lastTime)/dtime);
				ddisk = fixRange((disk*currTime-lastDisk[i]*lastTime)/dtime);
				deth = fixRange((eth*currTime-lastEth[i]*lastTime)/dtime);
				rdbw = fixRange((read-lastread[i])/dtime*Config.DISK_INVERSE_BANDWIDTH);
				rvbw = fixRange((recv-lastrecv[i])/dtime*Config.NETWORK_INVERSE_BANDWIDTH);
				sdbw = fixRange((send-lastsend[i])/dtime*Config.NETWORK_INVERSE_BANDWIDTH);
				
				lastCPU[i] = cpu;
				lastDisk[i] = disk;
				lastMem[i] = mem;
				lastEth[i] = eth;
				lastsend[i] = send;
				lastread[i] = read;
				lastrecv[i] = recv;
				
				model.getLogWriter().logNodeStatus(i, currTime, dcpu, ddisk, dmem, deth, rdbw, sdbw, rvbw);
			}
			lastTime=currTime;
		}
	}
	
}
