package simulation.log;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;

import desmoj.core.advancedModellingFeatures.Bin;
import desmoj.core.simulator.SimTime;
import desmoj.core.statistic.*;

import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;
import simulation.query.QueryResult;

public class Statistics {
	private ModelS model;
	
	public Histogram queryNumTerms;
	public Histogram queryNumResults;
	public Histogram queryIndexSize; 
	public Histogram queryTimes;
	
	public Statistics(ModelS model){
		this.model=model;
		queryNumTerms = new Histogram(model,"Number of terms per query",1.0,10.0,10,true,false);
		queryNumResults = new Histogram(model,"Number of results per query",0.0,1000000.0,20,true,false);
		queryIndexSize = new Histogram(model,"Total index size per query",0.0, 
				Config.NUMBER_OF_DOCUMENTS*Config.SIZE_OF_INDEX_DOCUMENT_ENTRY,20,true,false);
		queryTimes = new Histogram(model,"Query Times",0.0, Config.SIM_DURATION, 10, true,false);
	}
		
	public void addQueryTerms(int terms){
		queryNumTerms.update(terms);
	}
	
	public void addQueryResults(int results){
		queryNumResults.update(results);
	}
	
	public void addQueryIndexSize(double sz){
		queryIndexSize.update(sz);
	}
	
	public void addQueryTime(SimTime t){
		queryTimes.update(t.getTimeValue());
	}
	
	public void storeAll(){
		try {
			PrintWriter pwr = new PrintWriter(new FileWriter(ModelS.runName+".log"));
			double duration = SimTime.diff(model.currentTime(), model.resetAt()).getTimeValue();
			pwr.println(ModelS.runName+" : experiment results");
			pwr.println("Test Duration");
			pwr.println(duration);
			pwr.println("");
			pwr.println("Query Statistics");
			pwr.println("QPS \t" + model.getSolved() * 1000 / duration+"\n");
			pwr.println("Query time (ms)\t"+queryTimes.getMean()+" "+queryTimes.getMinimum()+" "+queryTimes.getMaximum());
			pwr.println("Query terms \t"+queryNumTerms.getMean()+" "+queryNumTerms.getMinimum()+" "+queryNumTerms.getMaximum());
			pwr.println("Query results \t"+queryNumResults.getMean()+" "+queryNumResults.getMinimum()+" "+queryNumResults.getMaximum());
			pwr.println("Query index size \t"+queryIndexSize.getMean()+" "+queryIndexSize.getMinimum()+" "+queryIndexSize.getMaximum());
			pwr.println("");
			
			ArrayList<Node> nodes = model.getNodes();
			pwr.println("Node Statistics: \t CPU load \t Disk load \t Eth load \t Mem load \t Disk seeks \t Data read \t Data sendt \t Data received");
			int i=0;
			for (Node node : nodes) {
				pwr.println((i++)+"\t"+
						node.getCPU().avgUsage()+"\t"+
						node.getDisk().avgUsage()+"\t"+
						node.getEth().avgUsage()+"\t"+
						node.getMemory().avgUsage()+"\t"+
						node.getSeeks()+"\t"+
						node.getDiskread()+"\t"+
						node.getEthsend()+"\t"+
						node.getEthrecv()
				);
			}
			pwr.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
