package simulation.micro;
import java.util.Random;

import simulation.model.Config;

public class HeapInterleaveTest {
	
	private static class PHeap{
		private int[] data;
		private int last, heapsize;
		
		
		public PHeap(int[] data){
			this.data = data;
			this.last = data.length-2;
			this.heapsize=data.length/2;
			buildHeap();
		}
		
		private void buildHeap(){
			for (int i=(heapsize-1)/2; i>=0; i--) heapify(i);
		}
		
		private void heapify(int p){
			int l = p*2+1;
			int r = p*2+2;
			
			if ( l >= heapsize) return;
			
			int n = (r >= heapsize) ? l : ((data[l*2] < data[r*2]) ? l : r);
			
			if (data[p*2] > data[n*2]) {
				int t1 = data[p*2];
				int t2 = data[p*2+1];
				
				data[p*2] = data[n*2];
				data[p*2+1] = data[n*2+1];
				data[n*2] = t1;
				data[n*2+1] = t2;
				
				heapify(n);
			}
		}
		
		public int getFirstValue(){
			/*
			for (int i=0;i<heapsize; i++){
				System.out.print(data[i*2]+":"+data[i*2+1]+" ");
			}
			System.out.println();
			*/
			return data[1];
		}
		
		public void setFirst(int key, int value){
			data[0] = key;
			data[1] = value;
			heapify(0);
		}
		
		public void removeFirst(){
			heapsize--;
			data[0] = data[heapsize*2];
			data[1] = data[heapsize*2+1];
			heapify(0);
		}
	}
	
	public static void test(int runs){
		double time=0.0;
		int listnum = 4;
		
		for (int j=0; j<runs;j++){
			int num = 200000;
			Random r = new Random(42);			
			int lists[][] = new int[listnum][num];

			int last[]={0,0,0,0,0,0,0,0};;
			for (int k=0; k<listnum; k++){
				for (int i=0; i<num; i+=2){
					last[k]+=r.nextInt(10);
					lists[k][i] = last[k];
					lists[k][i+1] = (int) ( r.nextDouble() * Config.NUMBER_OF_WORDS_PER_DOCUMENT);
				}
			}
			
			Runtime.getRuntime().freeMemory();
			long start = System.nanoTime();
			int reslist[] = new int[num*listnum];
			int p[] = {0,0,0,0,0,0,0,0};
			int pr = 0;
			
			int done = 0;
			
			int heap[] = new int[listnum*2]; 
			
			for (int i=0; i<listnum; i++){
				heap[i*2] = lists[i][0];
				heap[i*2+1] = i;
			}
			
			PHeap pheap = new PHeap(heap);
			
			int lastdoc=-1;
			int acc = 0;
			while (done<listnum){
				int listnr = pheap.getFirstValue();
				
				int docnr = lists[listnr][p[listnr]];
				int val = lists[listnr][p[listnr]+1];
				
				if (docnr==lastdoc){
					acc+=val;
				} else{
					if (lastdoc!=-1){
						reslist[pr++] = lastdoc;
						reslist[pr++] = acc;
					}
					lastdoc=docnr;
					acc=val;
				}
				p[listnr]+=2;
				
				if (p[listnr]<num){
					pheap.setFirst(lists[listnr][p[listnr]], listnr);
				} else {
					pheap.removeFirst();
					done++;
				}
			}
			
			/*
			for (int i=0; i<pr; i+=2){
				System.out.println(reslist[i] + ":" + reslist[i+1]+"+");
			}
			*/
			
			long end = System.nanoTime();
			time+=((double)(end-start))/(num*listnum/2) / (Math.log(listnum)/Math.log(2));
		}
		System.out.println(time/runs / 1000000 );
	}
	
	public static void main(String args[]){
		test(100);
		test(100);
		test(100);
		test(100);
		test(100);
	}		
}
