package simulation.micro;
import java.util.Random;

import simulation.model.Config;

public class HeapTopAndSortTest {
	private static class SimpleIndexResultHeap{
		private int[] data;
		private int heapsize;
		
		public SimpleIndexResultHeap(int[] data, int heapsize){
			this.data = data;
			this.heapsize = Math.min(heapsize, data.length/2);
						
			buildHeap();
			
			for (int i=heapsize; i<data.length/2; i++) {
				if (data[i*2+1] < data[1]){
					data[0] = data[i*2];
					data[1] = data[i*2+1];
					heapify(0);
				}
			}
		}
		
		private void buildHeap(){
			for (int i=(heapsize-1)/2; i>=0; i--) heapify(i);
		}
		
		private void heapify(int p){
			int l = p*2+1;
			int r = p*2+2;
			
			if ( l >= heapsize) return;
			
			int n = (r >= heapsize) ? l : ((data[l*2+1] < data[r*2+1]) ? l : r);
			
			if (data[p*2+1] > data[n*2+1]) {
				int t1 = data[p*2];
				int t2 = data[p*2+1];
				
				data[p*2] = data[n*2];
				data[p*2+1] = data[n*2+1];
				data[n*2] = t1;
				data[n*2+1] = t2;
				
				heapify(n);
			}
		}
	}
	
	
		
	public static void test(int runs){
		double time=0.0;

		for (int j=0; j<runs;j++){
			int num = 200000;
			Random r = new Random(42);			
			int lista[] = new int[num];			
			int lasta=0;
			for (int i=0; i<num; i+=2){
				lasta+=r.nextInt(10);
				
				lista[i] = lasta;
				lista[i+1] = (int) ( r.nextDouble() * Config.NUMBER_OF_WORDS_PER_DOCUMENT);	
			}
			
			Runtime.getRuntime().freeMemory();
			long start = System.nanoTime();
			
			SimpleIndexResultHeap rh = new SimpleIndexResultHeap(lista, 1024);
			
			long end = System.nanoTime();
			/*
			for (int i=0; i<num; i+=2){
				System.out.println(lista[i]);
			}
			*/
			time+=((double)(end-start))/(num/2 * Math.log(1024) / Math.log(2));
		}
		System.out.println(time/runs / 1000000);
	}
	
	public static void main(String args[]){
		//test2(1);
		test(1000);
		test(1000);
		test(1000);
		test(1000);
		test(1000);
	}
			
}
