package simulation.model;
import java.util.*;
import java.io.*;

public class Config {
	public static String DATAPATH = "/home/simonj/workspace/Sim2/data/";
	
	//algorithm choices
	public static final int LOCAL_INDEXING = 0;
	public static final int GLOBAL_INDEXING = 1;
	public static final int GLOBAL_PIPELINED_INDEXING = 2;
	public static final int HYBRID_INDEXING = 3;
	public static final int LOCAL_INDEXING_PARALLEL_MERGE = 4;
	public static final int GLOBAL_INDEXING_PARALLEL_MERGE = 5;
	public static final int GLOBAL_PIPELINED_INDEXING_PARALLEL_MERGE = 6;
	public static final int HYBRID_INDEXING_PARALLEL_MERGE = 7;
	
	//main test parameters, those are stored into 
	public static boolean SIMULATED_ONLY = true;
	public static boolean USE_AND_QUERIES = true;
	public static boolean DEMO_MODE = true;
	public static int INDEXING_MODE = -1;
	public static int NUMBER_OF_NODES = 4;
	public static int NUMBER_OF_CPUS_PER_NODE = 1;
	public static int NUMBER_OF_DISKS_PER_NODE = 1;
	public static int MAX_NUMBER_OF_QUERY_PROCESSES = 12;
	public static double CPU_FACTOR = 1.0;
	//number of queries to process before the non-demo benchmark
	public static int NUMBER_OF_WARMUP_QUERIES = 0;
	//maximum length of the experiment
	public static int SIM_DURATION = 10000; //
	//log sample time for demo run
	public static int LOG_SAMPLE_TIME = 10;
	
	//document collection constants
	public static int NUMBER_OF_RESULTS_REQUIRED = 1000;
	public static int NUMBER_OF_DOCUMENTS = 25000000;
	public static int NUMBER_OF_DOCUMENTS_STAT = 25205179;
	public static int NUMBER_OF_WORDS_PER_DOCUMENT = 10000; //used only for the microbenchmarking
	
	//memory contants
	public static int SIZE_OF_MEMORY_PER_NODE = 4096; //in BLOCKS
	public static int MEMORY_BLOCK_SIZE = 1048576; //in bytes
	public static int BUFFER_SIZE = 131072; //in bytes	
	
	//network performance constants
	public static double NETWORK_INVERSE_BANDWIDTH = 8E-6;
	public static double NETWORK_OVERHEAD = 0.08;
	
	//disk performance constants
	public static double DISK_INVERSE_BANDWIDTH = 1.25E-5;
	public static double DISK_SEEK_TIME = 8.5;
	public static double DISK_ROTATION_DELAY = 4.16;

	//cpu processing constants
	
	public static double COMPARSION_INSTRUCTION_TIME = 2.421848E-05 / CPU_FACTOR;
	public static double INTERLEAVE_TWO_LISTS_INSTRUCTION_TIME = 2.421848E-05 / CPU_FACTOR;
	public static double HEAP_INSTRUCTION_TIME = 5.896446E-07 / CPU_FACTOR;
	public static double HEAP_MULTIWAY_INTERLEAVEMERGE_TIME = 4.945716E-05 / CPU_FACTOR;
	public static double LOOKUP_TIME = 0.0105 /CPU_FACTOR;
	
	//data structure constants
	public static int QUERY_HEADER_LENGTH = 32;
	public static int SUBQUERY_HEADER_LENGTH = 32;
	public static int SUBQUERY_RESULT_HEADER_LENGTH = 32;
	public static int QUERY_RESULT_HEADER_LENGTH = 32;
	public static int SIZE_OF_INDEX_LOCATION_ENTRY = 5; //loc + 1byte descr
	public static int SIZE_OF_INDEX_DOCUMENT_ENTRY = 8; //docnum + occs
	public static int SIZE_OF_ACC_ENTRY = 8;
	public static int SIZE_OF_QUERY_ENTRY = 4;
	public static int SIZE_OF_RESULT_ENTRY = 8;
	//number of entries per chunk using the hybrid scheme
	public static int HYBRID_CHUNK_SIZE = 1024;	
	
	//cpu time slice for each process
	public static double CPU_SLICE_SIZE = 0.1;
	
	public static void loadProperties(String filename){
		try {
			Properties prop = new Properties();
			prop.load(new FileInputStream(filename));
			//DATAPATH = prop.getProperty("DATAPATH");
			SIMULATED_ONLY = Boolean.parseBoolean(prop.getProperty("SIMULATED_ONLY"));
			USE_AND_QUERIES = Boolean.parseBoolean(prop.getProperty("USE_AND_QUERIES"));
			DEMO_MODE = Boolean.parseBoolean(prop.getProperty("DEMO_MODE"));
			INDEXING_MODE = Integer.parseInt(prop.getProperty("INDEXING_MODE"));
			NUMBER_OF_NODES = Integer.parseInt(prop.getProperty("NUMBER_OF_NODES"));
			NUMBER_OF_CPUS_PER_NODE = Integer.parseInt(prop.getProperty("NUMBER_OF_CPUS_PER_NODE"));
			NUMBER_OF_DISKS_PER_NODE = Integer.parseInt(prop.getProperty("NUMBER_OF_DISKS_PER_NODE"));
			MAX_NUMBER_OF_QUERY_PROCESSES = Integer.parseInt(prop.getProperty("MAX_NUMBER_OF_QUERY_PROCESSES"));
			CPU_FACTOR = Double.parseDouble(prop.getProperty("CPU_FACTOR"));
			NUMBER_OF_WARMUP_QUERIES = Integer.parseInt(prop.getProperty("NUMBER_OF_WARMUP_QUERIES"));
			SIM_DURATION = Integer.parseInt(prop.getProperty("SIM_DURATION"));
			LOG_SAMPLE_TIME = Integer.parseInt(prop.getProperty("LOG_SAMPLE_TIME"));
			NUMBER_OF_RESULTS_REQUIRED = Integer.parseInt(prop.getProperty("NUMBER_OF_RESULTS_REQUIRED"));
			NUMBER_OF_DOCUMENTS = Integer.parseInt(prop.getProperty("NUMBER_OF_DOCUMENTS"));
			NUMBER_OF_DOCUMENTS_STAT = Integer.parseInt(prop.getProperty("NUMBER_OF_DOCUMENTS_STAT"));
			NUMBER_OF_WORDS_PER_DOCUMENT = Integer.parseInt(prop.getProperty("NUMBER_OF_WORDS_PER_DOCUMENT"));
			SIZE_OF_MEMORY_PER_NODE = Integer.parseInt(prop.getProperty("SIZE_OF_MEMORY_PER_NODE"));
			MEMORY_BLOCK_SIZE = Integer.parseInt(prop.getProperty("MEMORY_BLOCK_SIZE"));
			BUFFER_SIZE = Integer.parseInt(prop.getProperty("BUFFER_SIZE"));
			NETWORK_INVERSE_BANDWIDTH = Double.parseDouble(prop.getProperty("NETWORK_INVERSE_BANDWIDTH"));
			NETWORK_OVERHEAD =  Double.parseDouble(prop.getProperty("NETWORK_OVERHEAD"));
			DISK_INVERSE_BANDWIDTH =  Double.parseDouble(prop.getProperty("DISK_INVERSE_BANDWIDTH"));
			DISK_SEEK_TIME =  Double.parseDouble(prop.getProperty("DISK_SEEK_TIME"));
			DISK_ROTATION_DELAY =  Double.parseDouble(prop.getProperty("DISK_ROTATION_DELAY"));
			COMPARSION_INSTRUCTION_TIME =  Double.parseDouble(prop.getProperty("COMPARSION_INSTRUCTION_TIME"))/CPU_FACTOR;
			INTERLEAVE_TWO_LISTS_INSTRUCTION_TIME =  Double.parseDouble(prop.getProperty("INTERLEAVE_TWO_LISTS_INSTRUCTION_TIME"))/CPU_FACTOR;
			HEAP_INSTRUCTION_TIME = Double.parseDouble(prop.getProperty("HEAP_INSTRUCTION_TIME")) /CPU_FACTOR;
			HEAP_MULTIWAY_INTERLEAVEMERGE_TIME =Double.parseDouble(prop.getProperty("HEAP_MULTIWAY_INTERLEAVEMERGE_TIME")) /CPU_FACTOR;
			LOOKUP_TIME = Double.parseDouble(prop.getProperty("LOOKUP_TIME"))/CPU_FACTOR;
			QUERY_HEADER_LENGTH = Integer.parseInt(prop.getProperty("QUERY_HEADER_LENGTH"));
			SUBQUERY_HEADER_LENGTH = Integer.parseInt(prop.getProperty("SUBQUERY_HEADER_LENGTH"));
			SUBQUERY_RESULT_HEADER_LENGTH = Integer.parseInt(prop.getProperty("SUBQUERY_RESULT_HEADER_LENGTH"));
			QUERY_RESULT_HEADER_LENGTH =Integer.parseInt(prop.getProperty("QUERY_RESULT_HEADER_LENGTH")); 
			SIZE_OF_INDEX_LOCATION_ENTRY = Integer.parseInt(prop.getProperty("SIZE_OF_INDEX_LOCATION_ENTRY"));
			SIZE_OF_INDEX_DOCUMENT_ENTRY = Integer.parseInt(prop.getProperty("SIZE_OF_INDEX_DOCUMENT_ENTRY"));
			SIZE_OF_ACC_ENTRY = Integer.parseInt(prop.getProperty("SIZE_OF_ACC_ENTRY"));
			SIZE_OF_QUERY_ENTRY = Integer.parseInt(prop.getProperty("SIZE_OF_QUERY_ENTRY"));
			SIZE_OF_RESULT_ENTRY = Integer.parseInt(prop.getProperty("SIZE_OF_RESULT_ENTRY"));
			HYBRID_CHUNK_SIZE = Integer.parseInt(prop.getProperty("HYBRID_CHUNK_SIZE"));
			CPU_SLICE_SIZE = Double.parseDouble(prop.getProperty("CPU_SLICE_SIZE"));
		} catch (IOException e){
			e.printStackTrace();
		}
	}
}