package simulation.model;
import java.util.*;
import simulation.*;
import simulation.node.*;
import simulation.query.*;
import desmoj.core.dist.*;
import desmoj.core.simulator.*;


public class GeneratorProcess extends SimProcess{
	private IntDistUniform hitdist;
	private QueryLogReader qlr;
	
	
	public GeneratorProcess(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
		qlr = new QueryLogReader(owner);
		hitdist = new IntDistUniform(owner,"",0,Config.NUMBER_OF_DOCUMENTS-1,false,false);
	}
	
	@Override
	public void lifeCycle() {
		ModelS model = (ModelS) this.getModel();
		ArrayList<Node> nodes = model.getNodes();
		
		while (true){
			for (int i=0; i<nodes.size(); i++){
				Query q= new Query(model, "Query", false);
				int data[] = qlr.getNextQuery();
				int nterms= data.length / 2;
				for (int j=0; j<nterms; j++){
					int termid = data[2*j];
					double freq = ((double)data[2*j+1])/Config.NUMBER_OF_DOCUMENTS_STAT;
					if (Config.SIMULATED_ONLY)
						q.addTerm(termid, new SimulatedIndexHitList(freq));						
					else
						q.addTerm(termid, makeHitList(freq));
				}
				
				while(!model.scheduleQuery(q, i)){
					passivate();
				}
			
			}
		}
	}	
	
	public SimpleIndexHitList makeHitList(double freq){
		int numscored = (int)(freq * Config.NUMBER_OF_DOCUMENTS);
		int hits[] = new int[Config.NUMBER_OF_DOCUMENTS];
		for (int k=0; k< Config.NUMBER_OF_DOCUMENTS; k++ ){
			hits[k]=0;
		}
		for (int k=0; k< numscored; k++ ){
			int t=(int) hitdist.sample();
			while (hits[t]>0){
				t=(t+1) % hits.length;
			}
			hits[t]=1;
		}
		return new SimpleIndexHitList(hits);
	}
}
