package simulation.model;
import java.util.ArrayList;

import simulation.log.*;
import simulation.node.*;
import simulation.processes.*;
import simulation.query.*;
import desmoj.core.simulator.*;

public class ModelS extends Model {
	ArrayList<Node> nodes;
	Node broker;
	
	private LogWriter logwr; 
	private GeneratorProcess gen;
	private ResHandler res;
	private LogProcess logpr;
	private Statistics st;
	private boolean warmedup = false;
	private int scheduledqueries, solvedqueries, runningqueries;
	
	public static String runName;
	 
	
	public ModelS(Model owner, String name, boolean showInReport, boolean showIntrace) {
		super(owner, name, showInReport, showIntrace);
		runningqueries=0;
		scheduledqueries=0;
		solvedqueries=0;
	}
	
	public ArrayList<Node> getNodes(){
		return nodes;
	}
	public LogWriter getLogWriter(){
		return logwr;
	}
	
	public Node getBroker(){
		return broker;
	}
	
	@Override
	public String description() {
		return "\\,,/ o.O \\,,/";
	}

	@Override
	public void doInitialSchedules() {	
	}

	public ResHandler getResHandler(){
		return res;
	}
	
	public GeneratorProcess getGenerator(){
		return gen;
	}
	public boolean scheduleQuery(Query q, int nodenr){
		if (runningqueries<Config.MAX_NUMBER_OF_QUERY_PROCESSES){
			runningqueries++;
			scheduledqueries++;
			QueryProcess newqp;
			switch (Config.INDEXING_MODE){
				case (Config.HYBRID_INDEXING):
					newqp = new QueryProcessHD(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				case (Config.HYBRID_INDEXING_PARALLEL_MERGE):
					newqp = new QueryProcessHDPM(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				case (Config.GLOBAL_PIPELINED_INDEXING):
					newqp = new QueryProcessPL(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				case (Config.GLOBAL_PIPELINED_INDEXING_PARALLEL_MERGE):
					newqp = new QueryProcessPLPM(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				case (Config.GLOBAL_INDEXING):
					newqp = new QueryProcessGI(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				case (Config.GLOBAL_INDEXING_PARALLEL_MERGE):
					newqp = new QueryProcessGIPM(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				case (Config.LOCAL_INDEXING):
				default:
					newqp = new QueryProcessLI(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				case (Config.LOCAL_INDEXING_PARALLEL_MERGE):
					newqp = new QueryProcessLIPM(this, "QP"+nodenr, true, q, nodes.get(nodenr));
				break;
				
			}
			
			newqp.activate(new SimTime(0.0));
			if (Config.DEMO_MODE) 
				logwr.logSystemStatus(currentTime().getTimeValue(), scheduledqueries, solvedqueries);
			return true;
		} else {
			if (Config.DEMO_MODE)
				logwr.logSystemStatus(currentTime().getTimeValue(), scheduledqueries, solvedqueries);
			return false;
		}
	}
	
	public void querySolved(QueryProcess qp, Query q, QueryResult qr){
		
		st.addQueryTerms(q.getNumberOfTerms());
		st.addQueryIndexSize(q.getTotalIndexSize());
		st.addQueryResults(qr.getNumberOfDocuments());
		st.addQueryTime(SimTime.diff(currentTime(), qp.getStartTime()));
		
		if (!Config.DEMO_MODE && solvedqueries == Config.NUMBER_OF_WARMUP_QUERIES && !warmedup){
			//resetAllCounters
			solvedqueries=0;
			scheduledqueries=runningqueries;
			reset();
			for (Node node: nodes){
				node.resetCounters();
			}
			warmedup=true;
				
		}
		
		runningqueries--;
		solvedqueries++;
		if (runningqueries<Config.MAX_NUMBER_OF_QUERY_PROCESSES){
			gen.activate(new SimTime(0.0));
		}
	}
	
	public Statistics getStatistics(){
		return st;
	}
	
	public int getSolved(){
		return solvedqueries;
	}
	
	@Override
	public void init() {
		st = new Statistics(this);
		nodes=new ArrayList<Node>();
		res=new ResHandler();
		for (int i=0;i<Config.NUMBER_OF_NODES;i++){
			nodes.add(new Node(this,i, Config.NUMBER_OF_DISKS_PER_NODE,
					Config.NUMBER_OF_CPUS_PER_NODE, Config.SIZE_OF_MEMORY_PER_NODE));
		}
		if (Config.DEMO_MODE) {
			logwr = new LogWriter(this);
			logpr = new LogProcess(this, "Logger", false);
			logpr.activate(new SimTime(0.0));
		}
		broker = new Node(this, Config.NUMBER_OF_NODES+1, 1, 1, 51200);
		gen = new GeneratorProcess(this,"Generator",true);
		gen.activate(new SimTime(0.0));		
	}	
	
	public static void main(String args[]){
		if (args.length>0) runName = args[0];
		else runName = "default";
		
		Config.loadProperties(runName+".property");
		
		ModelS model = new ModelS(null, runName, true, true);
		Experiment exp = new Experiment(runName);
		model.connectToExperiment(exp);
		
		exp.setShowProgressBar(false);
		
		//exp.getReportManager().
		exp.stop(new SimTime(Config.SIM_DURATION));
		exp.tracePeriod(new SimTime(0.0), new SimTime(1000.0));
		exp.debugPeriod(new SimTime(0.0), new SimTime(0.0));
		exp.start();
		
		exp.report();
		exp.finish();
		
		if (Config.DEMO_MODE){
			model.logwr.renderNodeLog();
			model.logwr.renderProcLog();
		} else {
			model.st.storeAll();
		}
		
		System.out.println("results:"+Config.MAX_NUMBER_OF_QUERY_PROCESSES+"\t"+Config.NUMBER_OF_NODES+"\t"+(model.getSolved() * 1000 /  SimTime.diff(model.currentTime(), model.resetAt()).getTimeValue()) + "\t" +model.st.queryTimes.getMean());
	}	
}
