package simulation.processes;
import java.util.ArrayList;

import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;
import simulation.query.Query;
import simulation.query.QueryResult;
import simulation.query.SubQueryResult;


import desmoj.core.simulator.*;


public abstract class QueryProcess extends SimProcess{
	protected Query query;
	protected Node node;
	protected int subcnt;
	protected Queue resultQueue;
	protected QueryResult queryResult;
    protected SimTime startTime;
    
	public QueryProcess(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
	}
	
	public QueryProcess(Model owner, String name, boolean showInTrace, Query query, Node node) {
		super(owner, name, showInTrace);
		this.query=query;
		this.node=node;
		resultQueue = new Queue(owner, "RQ", false, false);
		queryResult = new QueryResult(owner, "QueryResult", false);
		startTime=currentTime();
		if (Config.DEMO_MODE) 
			((ModelS) owner).getLogWriter().logProcess(getName(), null, startTime.getTimeValue());
	}
	
	public void checkQueue(){
		SubQueryResult sqr;
		while ((sqr = (SubQueryResult)resultQueue.first())!=null){
			resultQueue.remove(sqr);
			queryResult.addResult(sqr);
			subcnt--;
		}
	}
	
	protected void ackQuery(SimProcess ackpro, SubQueryResult sqr){
		if (sqr!=null) resultQueue.insert(sqr);
		else subcnt--;
		
		if (this.isScheduled()){
			if ( SimTime.isSmallerOrEqual(this.currentTime(), this.scheduledAt())){
				this.reActivate(new SimTime(0.0));
			} else {
				System.out.println("bugbug");
			}
		} else if ( this.isBlocked()){
			//do nothing? got lucky?
		} else {
			this.activate(new SimTime(0.0));
		}
	}
	
	public Query getQuery() {
		return query;
	}

	public Node getNode() {
		return node;
	}

	@Override
	public void lifeCycle() {
	}
	
	public SimTime getStartTime(){
		return startTime;
	}
}