package simulation.processes;
import java.util.*;

import simulation.log.LogWriter;
import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;
import simulation.query.IndexHitList;
import simulation.query.IndexTools;
import simulation.query.Query;
import simulation.query.SubQuery;
import simulation.query.SubQueryResult;



import desmoj.core.simulator.*;


public class QueryProcessGIPM extends QueryProcess {
	private int totalentries = 0;
	private ArrayList<IndexHitList> hitlists = new ArrayList<IndexHitList>();
	
	public QueryProcessGIPM(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
	}

	public QueryProcessGIPM(Model owner, String name, boolean showInTrace, Query query, Node node) {
		super(owner, name, showInTrace, query, node);
	}
	
	@Override
	public void checkQueue(){
		SubQueryResult sqr;
		
		while ((sqr = (SubQueryResult)resultQueue.first())!=null){
			resultQueue.remove(sqr);
			queryResult.addResult(sqr);
			
			IndexHitList subreslist= sqr.getHitList();		
			totalentries += subreslist.getNumDocs();
			hitlists.add(subreslist);
			
			subcnt--;
		}
	}
	
	@Override
	public void lifeCycle() {
		ModelS ms = (ModelS) getModel();
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.START, 
				LogWriter.OTHER);
		
		node.transfer(ms.getBroker(), node, this, Config.QUERY_HEADER_LENGTH + query.getNumberOfTerms() *
				Config.SIZE_OF_QUERY_ENTRY);
		
		//node.lookupTerms(this, query.getNumberOfTerms());
		node.lookupTerms(this, query.getNumberOfTerms());
		
		//no results for and query
		if ( query.getTerms().get(0) < 0){
			if (Config.USE_AND_QUERIES) {
				node.transfer(node, ms.getBroker(), this, Config.QUERY_RESULT_HEADER_LENGTH);
				if (Config.DEMO_MODE)
					ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(),
						LogWriter.FINISH, LogWriter.OTHER);
				ms.querySolved(this, query, queryResult);
				//System.out.println(query.getNumberOfTerms()+":"+queryResult.getNumberOfDocuments());
				return;
			} else {
				query.eliminateNonExistingTerms();
			}
		}
		ArrayList<Node> nodes = ms.getNodes();
		ArrayList<SubQuery> subs = query.partitionByTerm();
		subcnt = subs.size();
		
		//allocate enough memory
		int memreq = Config.NUMBER_OF_DOCUMENTS * Config.SIZE_OF_ACC_ENTRY * (subcnt+1);
		node.malloc(this,memreq);

		for (SubQuery subQuery : subs) {
			int destNode = subQuery.getNodeNumber();
			SubQueryProcessGIPM sp = new SubQueryProcessGIPM(ms,"SQP"+destNode,true,
					this, subQuery , nodes.get(destNode));
			sp.activate(new SimTime(0.0));
		}
						
		//result trick
		do{
			passivate();	
			checkQueue();
		} while (subcnt>0);
		
		//postprocess
		queryResult.setHitList(IndexTools.interleaveEntries(hitlists));
		
		node.heapInterleave(this, totalentries, hitlists.size());
		node.heapTopAndSort(this, queryResult.getNumberOfDocuments(), Config.NUMBER_OF_RESULTS_REQUIRED);
		
		node.transfer(node, ms.getBroker(), this, Config.QUERY_RESULT_HEADER_LENGTH +
				Math.min(Config.NUMBER_OF_RESULTS_REQUIRED, queryResult.getNumberOfDocuments()) *
				Config.SIZE_OF_RESULT_ENTRY);		
		node.free(memreq);
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.FINISH, 
				LogWriter.OTHER);
		ms.querySolved(this, query, queryResult);
		//System.out.println(query.getNumberOfTerms()+":"+queryResult.getNumberOfDocuments());
	}
}
