package simulation.processes;
import java.util.ArrayList;

import simulation.log.LogWriter;
import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;
import simulation.query.Query;
import simulation.query.SubQuery;
import simulation.query.SubQueryResult;


import desmoj.core.simulator.*;


public class QueryProcessLI extends QueryProcess {
	private double numres = 0;
	
	public QueryProcessLI(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
	}

	public QueryProcessLI(Model owner, String name, boolean showInTrace, Query query, Node node) {
		super(owner, name, showInTrace, query, node);
	}
	
	@Override
	public void checkQueue(){
		SubQueryResult sqr;
		while ((sqr = (SubQueryResult) resultQueue.first())!=null){
			resultQueue.remove(sqr);
			queryResult.addResult(sqr);
			numres+=Math.min(Config.NUMBER_OF_RESULTS_REQUIRED, sqr.getNumberOfDocuments());
			subcnt--;
		}
	}
	
	@Override
	public void lifeCycle() {
		ModelS ms = (ModelS) getModel();
		
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.START, 
				LogWriter.OTHER);
				
		node.transfer(ms.getBroker(), node, this, Config.QUERY_HEADER_LENGTH + query.getNumberOfTerms() *
				Config.SIZE_OF_QUERY_ENTRY);
			
		ArrayList<Node> nodes = ms.getNodes();
		ArrayList<SubQuery> subs = query.partitionByDoc();
		subcnt = subs.size();
		
		//allocate enough memory
		int memreq = Config.NUMBER_OF_RESULTS_REQUIRED * subcnt * Config.SIZE_OF_RESULT_ENTRY;
		node.malloc(this, memreq);
		
		
		for (SubQuery subQuery : subs) {
			int destNumber = subQuery.getNodeNumber();
			SubQueryProcess newsqp = new SubQueryProcessLI(ms,"SQP"+destNumber,true,
					this, subQuery , nodes.get(destNumber));
			newsqp.activate(new SimTime(0.0));	
		}
		
		do{
			passivate();	
			checkQueue();
		} while (subcnt>0);
		
		
		queryResult.combineResults();

		//FIXME: there is an alternative here! progressive heapmerge

		node.heapMerge(this, (int) Math.min(Config.NUMBER_OF_RESULTS_REQUIRED, numres), Config.NUMBER_OF_NODES);
		
		node.heapTopAndSort(this, (int) Math.min(Config.NUMBER_OF_RESULTS_REQUIRED, numres),
				Config.NUMBER_OF_RESULTS_REQUIRED);
		
		node.transfer(node, ms.getBroker(), this, Config.QUERY_RESULT_HEADER_LENGTH + 
				Math.min(Config.NUMBER_OF_RESULTS_REQUIRED, queryResult.getNumberOfDocuments()) *
				Config.SIZE_OF_RESULT_ENTRY);
		
		node.free(memreq);
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.FINISH, 
				LogWriter.OTHER);
		
		ms.querySolved(this, query, queryResult);
		//System.out.println(query.getNumberOfTerms()+":"+queryResult.getNumberOfDocuments());
	}
}
