package simulation.processes;
import java.util.*;

import simulation.log.LogWriter;
import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;
import simulation.query.IndexHitList;
import simulation.query.IndexTools;
import simulation.query.Query;
import simulation.query.SubQuery;




import desmoj.core.simulator.*;


public class QueryProcessPL extends QueryProcess {
	
	public QueryProcessPL(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
	}

	public QueryProcessPL(Model owner, String name, boolean showInTrace, Query query, Node node) {
		super(owner, name, showInTrace, query, node);
	}
	
	@Override
	public void checkQueue(){}
	
	@Override
	public void lifeCycle() {
		
		ModelS ms = (ModelS) getModel();
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.START, 
				LogWriter.OTHER);
		
		node.transfer(ms.getBroker(), node, this, Config.QUERY_HEADER_LENGTH + query.getNumberOfTerms()
				* Config.SIZE_OF_QUERY_ENTRY);
				
		node.lookupTerms(this, query.getNumberOfTerms());
		if ( query.getTerms().get(0) < 0){
			if (Config.USE_AND_QUERIES) {
				node.transfer(node, ms.getBroker(), this, Config.QUERY_RESULT_HEADER_LENGTH);
				if (Config.DEMO_MODE)
					ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(),
						LogWriter.FINISH, LogWriter.OTHER);
				ms.querySolved(this, query, queryResult);
				//System.out.println(query.getNumberOfTerms()+":"+queryResult.getNumberOfDocuments());
				return;
			} else {
				query.eliminateNonExistingTerms();
			}
		}
		
		ArrayList<Node> nodes = ms.getNodes();
		ArrayList<SubQuery> subs = query.partitionByTerm();
		subcnt=subs.size();
		
		int subcntc = subcnt;
		Node prev = null;
		Node curr = node;
		int memreq = 2*Config.NUMBER_OF_DOCUMENTS*Config.SIZE_OF_ACC_ENTRY;
		
		
		curr.malloc(this, memreq);
		IndexHitList reslist=null;
		
		if (subcnt==0){
			queryResult.setHitList(reslist);
			curr.transfer(curr, ms.getBroker(), this, Config.QUERY_RESULT_HEADER_LENGTH);
		} else {	
			while(true){
				int bestcand=0;
				for (int i=1; i < subcnt; i++){
					//TODO: fix the heuristic part here!
					if (subs.get(bestcand).getLowestScore()>subs.get(i).getLowestScore()){
						bestcand=i;
					}
				}
				prev=curr;
				curr=nodes.get(subs.get(bestcand).getNodeNumber());
		
				SubQuery subquery=subs.get(bestcand);
			
				if (prev!=curr){
					prev.transfer(curr, prev, this, Config.SUBQUERY_HEADER_LENGTH*subcntc + query.getNumberOfTerms() *
						Config.SIZE_OF_QUERY_ENTRY+(reslist!=null?reslist.getNumDocs()*Config.SIZE_OF_ACC_ENTRY:0));
					prev.free(memreq);
					curr.malloc(this, memreq);
				}
			
				node.lookupTerms(this, subquery.getNumberOfTerms());
				int memreq2 = Config.NUMBER_OF_DOCUMENTS * Config.SIZE_OF_ACC_ENTRY;
				curr.malloc(this, memreq2);
				
				for (Integer termid : subquery.getSortedTermList()) {
					IndexHitList termlist=subquery.getHitList(termid);
					node.fetch(this, subquery.getNumberOfDocuments(termid) * Config.SIZE_OF_INDEX_DOCUMENT_ENTRY);
				
					node.interleaveTwoLists(this, termlist.getNumDocs()+(reslist!=null?reslist.getNumDocs():0));
					if (reslist==null)
						reslist = termlist;
					else 
						reslist = IndexTools.interleaveEntries(reslist, termlist);
				}
				
				curr.free(memreq2);
				subs.remove(bestcand);
			
				if (--subcnt==0){
					//postprocess
					queryResult.setHitList(reslist);
				
					curr.heapTopAndSort(this, queryResult.getNumberOfDocuments(), Config.NUMBER_OF_RESULTS_REQUIRED);
					curr.transfer(curr, ms.getBroker(), this, Config.QUERY_RESULT_HEADER_LENGTH + 
							Math.min(Config.NUMBER_OF_RESULTS_REQUIRED,queryResult.getNumberOfDocuments()) *
							Config.SIZE_OF_RESULT_ENTRY);
					
					break;
				}
			}
		}
		
		curr.free(memreq);
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.FINISH, 
				LogWriter.OTHER);
		
		ms.querySolved(this, query, queryResult);
		//System.out.println(query.getNumberOfTerms()+":"+queryResult.getNumberOfDocuments());
	}
}
