package simulation.processes;
import simulation.log.LogWriter;
import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;
import simulation.query.IndexHitList;
import simulation.query.IndexTools;
import simulation.query.SubQuery;
import simulation.query.SubQueryResult;
import desmoj.core.simulator.*;

public class SubQueryProcessLI extends SubQueryProcess{
		
	public SubQueryProcessLI(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
	}
	
	public SubQueryProcessLI(Model owner, String name, boolean showInTrace,
			 QueryProcess queryprocess, SubQuery subquery, Node node) {
		super(owner, name, showInTrace, queryprocess, subquery, node);
	}
	
	@Override
	public void lifeCycle() {
		//example
		Node othernode = queryprocess.getNode();
		ModelS ms = (ModelS) getModel();
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.START, 
				LogWriter.OTHER);
		//transfer data
		node.transfer(othernode, node, this, Config.SUBQUERY_HEADER_LENGTH + Config.SIZE_OF_QUERY_ENTRY * 
				subquery.getQuery().getNumberOfTerms());
		
		node.lookupTerms(this, subquery.getQuery().getNumberOfTerms());
		
		//no results for and query
		if ( subquery.getTerms().get(0) < 0){
			if (Config.USE_AND_QUERIES) {
				node.transfer(node, othernode, this, Config.SUBQUERY_RESULT_HEADER_LENGTH);
				queryprocess.ackQuery(this, null);
				if (Config.DEMO_MODE)
					ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.FINISH, 
						LogWriter.OTHER);
				return;
			} else {
				subquery.eliminateNonExistingTerms();
			}
		}
		
		//allocate		
		int memreq = Config.NUMBER_OF_DOCUMENTS / Config.NUMBER_OF_NODES * (Config.SIZE_OF_INDEX_DOCUMENT_ENTRY + Config.SIZE_OF_ACC_ENTRY);
		node.malloc(this, memreq);

		IndexHitList reslist=null;
		for (Integer termid : subquery.getSortedTermList()){
			IndexHitList termlist=subquery.getHitList(termid);
			node.fetch(this, subquery.getNumberOfDocuments(termid) * Config.SIZE_OF_INDEX_DOCUMENT_ENTRY);
			
			node.interleaveTwoLists(this, termlist.getNumDocs()+(reslist!=null?reslist.getNumDocs():0));
			if (reslist == null)
				reslist = termlist;
			else 
				reslist = IndexTools.interleaveEntries(reslist, termlist);
		}
			
		//postprocess
		SubQueryResult subqueryresult = new SubQueryResult(getModel(), "SubQueryResult", false, reslist);
		
			node.heapTopAndSort(this, subqueryresult.getNumberOfDocuments(), Config.NUMBER_OF_RESULTS_REQUIRED);
		
		
		//transfer back:
		node.transfer(node, othernode, this, Config.SUBQUERY_RESULT_HEADER_LENGTH + Config.SIZE_OF_RESULT_ENTRY *
				Math.min(Config.NUMBER_OF_RESULTS_REQUIRED, subqueryresult.getNumberOfDocuments()));
		
		//free memory
		node.free(memreq);
		
		//ack query
		queryprocess.ackQuery(this, subqueryresult);
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.FINISH, 
				LogWriter.OTHER);
	}

}