package simulation.processes;
import java.util.ArrayList;

import simulation.log.LogWriter;
import simulation.model.Config;
import simulation.model.ModelS;
import simulation.node.Node;
import simulation.query.IndexHitList;
import simulation.query.IndexTools;
import simulation.query.SubQuery;
import simulation.query.SubQueryResult;


import desmoj.core.simulator.*;

public class SubQueryProcessLIPM extends SubQueryProcess{
		
	public SubQueryProcessLIPM(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
	}
	
	public SubQueryProcessLIPM(Model owner, String name, boolean showInTrace,
			 QueryProcess queryprocess, SubQuery subquery, Node node) {
		super(owner, name, showInTrace, queryprocess, subquery, node);
	}
	
	@Override
	public void lifeCycle() {
		//example
		Node othernode = queryprocess.getNode();
		ModelS ms = (ModelS) getModel();
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.START, 
				LogWriter.OTHER);
		//transfer data
		node.transfer(othernode, node, this, Config.SUBQUERY_HEADER_LENGTH + Config.SIZE_OF_QUERY_ENTRY * 
				subquery.getNumberOfTerms());
		
		node.lookupTerms(this, subquery.getQuery().getNumberOfTerms());
		
		if ( subquery.getTerms().get(0) < 0){
			if (Config.USE_AND_QUERIES) {
				node.transfer(node, othernode, this, Config.SUBQUERY_RESULT_HEADER_LENGTH);
				queryprocess.ackQuery(this, null);
				if (Config.DEMO_MODE)
					ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.FINISH, 
						LogWriter.OTHER);
				return;
			} else {
				subquery.eliminateNonExistingTerms();
			}
		}
		//allocate		
		int memreq = Config.NUMBER_OF_DOCUMENTS / Config.NUMBER_OF_NODES * 
			(Config.SIZE_OF_INDEX_DOCUMENT_ENTRY * subquery.getQuery().getNumberOfTerms() + 
			Config.SIZE_OF_ACC_ENTRY);
		
		node.malloc(this, memreq);

		ArrayList<IndexHitList> hitlists = new ArrayList<IndexHitList>();
		int totalentries = 0;
		for (Integer termid : subquery.getSortedTermList()){
			IndexHitList termlist=subquery.getHitList(termid);
			node.fetch(this, subquery.getNumberOfDocuments(termid) * Config.SIZE_OF_INDEX_DOCUMENT_ENTRY);
			
			hitlists.add(termlist);
			totalentries += termlist.getNumDocs();
		}
			
		IndexHitList reslist = IndexTools.interleaveEntries(hitlists);

		//simulates merging extraction from heap
		node.heapInterleave(this, totalentries, hitlists.size());
		node.heapTopAndSort(this, reslist.getNumDocs(), Config.NUMBER_OF_RESULTS_REQUIRED);
		
		//postprocess
		SubQueryResult subqueryresult = new SubQueryResult(getModel(), "SubQueryResult", false, reslist);

		//transfer back:
		node.transfer(node, othernode, this, Config.SUBQUERY_RESULT_HEADER_LENGTH + Config.SIZE_OF_RESULT_ENTRY *
				Math.min(Config.NUMBER_OF_RESULTS_REQUIRED, subqueryresult.getNumberOfDocuments()));
		
		//free memory
		node.free(memreq);
		
		//ack query
		queryprocess.ackQuery(this, subqueryresult);
		if (Config.DEMO_MODE)
			ms.getLogWriter().logProcessStatus(this.getName(), currentTime().getTimeValue(), LogWriter.FINISH, 
				LogWriter.OTHER);
	}

}