package simulation.query;
import java.util.*;
import java.io.*;

import simulation.model.Config;

public class IndexTools {
	public static IndexHitList mergeEntries(ArrayList<IndexHitList> entries){
		if (Config.SIMULATED_ONLY){
			if (entries.size()==0) return new SimulatedIndexHitList(0.0);
			SimulatedIndexHitList first = (SimulatedIndexHitList)entries.get(0);
			SimulatedIndexHitList tmp;
			double acc = (first!=null) ? first.getDocFrequency() : 0;
			for(int i=1; i<entries.size(); i++){
				tmp=(SimulatedIndexHitList) entries.get(i);
				if (tmp!=null) acc += tmp.getDocFrequency();
			}				 
			return new SimulatedIndexHitList(acc);
		} else {
			SimpleIndexHitList res = new SimpleIndexHitList();
			int nums = entries.size();
			if (nums==1) return entries.get(0);
			int scores[] = new int[Config.NUMBER_OF_DOCUMENTS];
			for (int i=0; i<Config.NUMBER_OF_DOCUMENTS; i++){
				scores[i]=0;
			}
			
			for (IndexHitList entry : entries) {
				ArrayList<SimpleIndexHit> hitlist = ((SimpleIndexHitList)entry).getHitList();
				for (SimpleIndexHit record : hitlist) {
					scores[record.getDocid()]=record.getHits();
				}
			}
		
			for (int i=0; i<Config.NUMBER_OF_DOCUMENTS; i++){
				if (scores[i]>0) res.addHits(i, scores[i]);
			}
			return res;				
		}
	}
	
	public static IndexHitList interleaveEntries(ArrayList<IndexHitList> entries){
		if (Config.SIMULATED_ONLY){
			if (Config.USE_AND_QUERIES){
				if (entries.size()==0) return new SimulatedIndexHitList(0.0);
				double acc = ((SimulatedIndexHitList)entries.get(0)).getDocFrequency();
				for(int i=1; i<entries.size(); i++){
					acc *= ((SimulatedIndexHitList) entries.get(i)).getDocFrequency();
				}				 
				return new SimulatedIndexHitList(acc);
			} else {
				if (entries.size()==0) return new SimulatedIndexHitList(0.0);
				double acc = 1 - ((SimulatedIndexHitList)entries.get(0)).getDocFrequency();
				for(int i=1; i<entries.size(); i++){
					acc *= 1 - ((SimulatedIndexHitList) entries.get(i)).getDocFrequency();
				}				 
				return new SimulatedIndexHitList(1-acc);
			}

		} else {
			SimpleIndexHitList res = new SimpleIndexHitList();
			if (Config.USE_AND_QUERIES){
				int nums = entries.size();
				if (nums==1) return entries.get(0);
				int scores[] = new int[Config.NUMBER_OF_DOCUMENTS];
				int cnts[] = new int[Config.NUMBER_OF_DOCUMENTS];
				for (int i=0; i<Config.NUMBER_OF_DOCUMENTS; i++){
					scores[i]=0;
					cnts[i]=0;
				}
				
				for (IndexHitList entry : entries) {
					ArrayList<SimpleIndexHit> hitlist = ((SimpleIndexHitList)entry).getHitList();

					for (SimpleIndexHit record : hitlist) {
						if (scores[record.getDocid()] > record.getHits())
							scores[record.getDocid()]= record.getHits();
						cnts[record.getDocid()]++;
					}
				}
		
				for (int i=0; i<Config.NUMBER_OF_DOCUMENTS; i++){
					if (scores[i]>0 && cnts[i]==nums) res.addHits(i, scores[i]);
				}
				return res;
			} else {
				int nums = entries.size();
				if (nums==1) return entries.get(0);
				int scores[] = new int[Config.NUMBER_OF_DOCUMENTS];
				for (int i=0; i<Config.NUMBER_OF_DOCUMENTS; i++){
					scores[i]=0;
				}
				
				for (IndexHitList entry : entries) {
					ArrayList<SimpleIndexHit> hitlist = ((SimpleIndexHitList)entry).getHitList();

					for (SimpleIndexHit record : hitlist) {
						scores[record.getDocid()]+=record.getHits();
					}
				}
		
				for (int i=0; i<Config.NUMBER_OF_DOCUMENTS; i++){
					if (scores[i]>0) res.addHits(i, scores[i]);
				}
				return res;				
			}
		}
	}
	
	
	public static IndexHitList interleaveEntries(IndexHitList lista, IndexHitList listb){
		if (lista==null) return listb;
		if (listb==null) return lista;
		if (Config.SIMULATED_ONLY){
			if (Config.USE_AND_QUERIES)
				return new SimulatedIndexHitList(
						((SimulatedIndexHitList) lista).getDocFrequency() *
						((SimulatedIndexHitList) listb).getDocFrequency());
			else
			  return new SimulatedIndexHitList( 1 - 
					 (1-((SimulatedIndexHitList) lista).getDocFrequency()) *
					 (1-((SimulatedIndexHitList) listb).getDocFrequency()));
		} else {
				SimpleIndexHitList res = new SimpleIndexHitList();
				ArrayList<SimpleIndexHit> hitsa = ((SimpleIndexHitList)lista).getHitList();
				ArrayList<SimpleIndexHit> hitsb = ((SimpleIndexHitList)listb).getHitList();
				int as = hitsa.size();
				int bs = hitsb.size();
				int ap=0, bp=0;
				while (ap<as && bp<bs ){
					SimpleIndexHit hita = hitsa.get(ap);
					SimpleIndexHit hitb = hitsb.get(bp);
					int hits1 = hita.getHits();
					int hits2 = hita.getHits();
					
					if ( hita.compareTo(hitb) < 0 ){
						if (!Config.USE_AND_QUERIES) res.addHits(hita.getDocid(), hits1);
						ap++;
					} else if ( hita.compareTo(hitb) > 0){
						if (!Config.USE_AND_QUERIES) res.addHits(hitb.getDocid(), hits2);
						bp++;
					} else {
						if (Config.USE_AND_QUERIES) res.addHits(hitb.getDocid(), hits1<hits2?hits1:hits2);
						else res.addHits(hitb.getDocid(), hits1+hits2);
						ap++; bp++;
					}
				}
				if (!Config.USE_AND_QUERIES) {
					if (ap==as){
						SimpleIndexHit hitb;
						for (; bp < bs; bp++){
							hitb = hitsb.get(bp);
							res.addHits(hitb.getDocid(), hitb.getHits());	
						}
					} else if (bp==bs){
						SimpleIndexHit hita;
						for (; ap < as; ap++){
							hita = hitsa.get(ap);
							res.addHits(hita.getDocid(), hita.getHits());	
						}
					}
				}
			return res;
		}
	}

public static IndexHitList mergeEntries(IndexHitList lista, IndexHitList listb){
	if (lista==null) return listb;
	if (listb==null) return lista;
	if (Config.SIMULATED_ONLY){
			return new SimulatedIndexHitList(
					((SimulatedIndexHitList) lista).getDocFrequency() +
					((SimulatedIndexHitList) listb).getDocFrequency());
	} else {
			SimpleIndexHitList res = new SimpleIndexHitList();
			ArrayList<SimpleIndexHit> hitsa = ((SimpleIndexHitList)lista).getHitList();
			ArrayList<SimpleIndexHit> hitsb = ((SimpleIndexHitList)listb).getHitList();
			int as = hitsa.size();
			int bs = hitsb.size();
			int ap=0, bp=0;
			while (ap<as && bp<bs ){
				SimpleIndexHit hita = hitsa.get(ap);
				SimpleIndexHit hitb = hitsb.get(bp);
				int hits1 = hita.getHits();
				int hits2 = hita.getHits();
				
				if ( hita.compareTo(hitb) < 0 ){
					res.addHits(hita.getDocid(), hits1);
					ap++;
				} else if ( hita.compareTo(hitb) > 0){
					res.addHits(hitb.getDocid(), hits2);
					bp++;
				}
			}
			if (ap==as){
				SimpleIndexHit hitb;
				for (; bp < bs; bp++){
					hitb = hitsb.get(bp);
					res.addHits(hitb.getDocid(), hitb.getHits());	
				}
			} else if (bp==bs){
				SimpleIndexHit hita;
				for (; ap < as; ap++){
					hita = hitsa.get(ap);
					res.addHits(hita.getDocid(), hita.getHits());	
				}
			}			
		return res;
	}
}
}