package simulation.query;
import java.util.*;

import desmoj.core.simulator.*;
import simulation.*;
import simulation.model.Config;

public class QueryResult extends Entity{
	private ArrayList<SubQueryResult> results;
	private IndexHitList hitlist;
	
	public QueryResult(Model owner, String name, boolean showInTrace) {
		super(owner, name, showInTrace);
		results = new ArrayList<SubQueryResult>();
	}
	
	public void addResult(SubQueryResult result){
		results.add(result);
	}
	
	public void combineResults(){
		if (hitlist!=null) return;
		ArrayList<IndexHitList> hitlists = new ArrayList<IndexHitList>();
		for (SubQueryResult result : results) {
			hitlists.add(result.getHitList());
		}
		if (Config.INDEXING_MODE == Config.LOCAL_INDEXING ||
			Config.INDEXING_MODE == Config.LOCAL_INDEXING_PARALLEL_MERGE
		) hitlist = IndexTools.mergeEntries(hitlists);
		else hitlist=IndexTools.interleaveEntries(hitlists);
	}
	
	public void setHitList(IndexHitList hitlist){
		this.hitlist=hitlist;
	}
	public IndexHitList getHitList(){
		if (hitlist==null)
			if (results.size()>0) combineResults();
			else return null;
		return hitlist;
	}
	
	public int getNumberOfDocuments(){
		if (hitlist==null) combineResults();
		return hitlist.getNumDocs();
	}
}
