package simulation.query;
import java.util.*;

import desmoj.core.simulator.*;

public class SubQuery extends Entity{
	private HashMap<Integer, IndexHitList> termHits;
	private int nodeNumber;
	private Query query;

	public SubQuery(Model owner, String name, boolean showInTrace){
		super(owner, name, showInTrace);
		termHits = new HashMap<Integer, IndexHitList>();
	}
	
	public SubQuery(Model owner, String name, boolean showInTrace, int nodeNumber, Query query){
		super(owner, name, showInTrace);
		termHits = new HashMap<Integer, IndexHitList>();
		this.nodeNumber = nodeNumber;
		this.query = query;
	}
	
	public int getNumberOfTerms(){
		return termHits.size();
	}
	
	public ArrayList<Integer> getTermList(){
		return new ArrayList<Integer>(termHits.keySet());
	}

	public IndexHitList getHitList(int termid){
		return termHits.get(termid);
	}
	
	public int getNumberOfDocuments(int termid){
		return termHits.get(termid).getNumDocs();
	}
	
	public int getNodeNumber(){
		return nodeNumber;
	}

	public void addTerm(int termid, IndexHitList hitlist){
		termHits.put(termid, hitlist);
	}

	/**
	 * used only by SimulatedIndexHitList
	 */
	public void addTermFreq(int termid, double freq) {
		IndexHitList hitlist = termHits.get(termid);
		if (hitlist == null){
			hitlist = new SimulatedIndexHitList();
			termHits.put(termid, hitlist);
		}		
		((SimulatedIndexHitList) hitlist).addHits(freq);
	}
	
	/**
	 * used only by SimpleIndexHitList
	 */
	public void addTermHits(int termid, int docid, int hits) {
		IndexHitList hitlist = termHits.get(termid);
		if (hitlist == null){
			hitlist = new SimpleIndexHitList();
			termHits.put(termid, hitlist);
		}		
		((SimpleIndexHitList) hitlist).addHits(docid, hits);
	}
	
	public SubQueryResult getSubQueryResult(){
		return new SubQueryResult(getModel(), "SubQueryResult", false, 
				new ArrayList<IndexHitList>(termHits.values()));
	}

	public int getLowestScore() {
		int lowest = Integer.MAX_VALUE;
		for (IndexHitList hitlist : termHits.values()) {
			if (hitlist.getNumDocs() < lowest ) lowest = hitlist.getNumDocs();
		}
		return lowest;
	}
	
	public Query getQuery(){
		return query;
	}

	public ArrayList<Integer> getTerms(){
		ArrayList<Integer> res = new ArrayList<Integer>(termHits.keySet());
		Collections.sort(res);
		return res;
	}
	
	public void eliminateNonExistingTerms(){
		ArrayList<Integer> keyset = new ArrayList<Integer>(termHits.keySet());
		int i=0;
		while (i<keyset.size()){
			if (keyset.get(i) < 0){
				termHits.remove(keyset.get(i));
				keyset.remove(i);
			} else {
				i++;
			}
		}
	}
	
	public ArrayList<Integer> getSortedTermList() {
		// TODO Auto-generated method stub
		int cnt = termHits.size();
		if (cnt==1){
			return new ArrayList<Integer>(termHits.keySet());
		}
		ArrayList<Integer> res = new ArrayList<Integer>(cnt);
		
		ArrayList<Pair> vals = new ArrayList<Pair>(termHits.size());
		for (Integer key : termHits.keySet()){
			vals.add(new Pair(key, getNumberOfDocuments(key)));
		}
		Collections.sort(vals);
		
		for (int i=0; i<cnt; i++){
			res.add(vals.get(i).getId());
		}
		
		return res;
	}
	
	private class Pair implements Comparable<Pair>{
		private int id, value;
		
		public Pair(int id, int value){
			this.id = id;
			this.value = value;
		}
		
		public int compareTo(Pair p){
			if (value > p.value) return 1;
			else if (value < p.value) return -1;
			else return 0;
		}
		
		public int getId(){
			return id;
		}
	}
}
